package com.jenny.magic.entities;

import net.minecraft.util.Mth;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.Vec3;
import org.jetbrains.annotations.NotNull;

public abstract class BaseWandProjectile extends AbstractArrow {
    protected BaseWandProjectile(EntityType<? extends AbstractArrow> pEntityType, Level pLevel) {
        super(pEntityType, pLevel);
    }

    public void shootFromRotation(Entity pShooter, float pX, float pY, float pZ, float pVelocity) {
        float f = -Mth.sin(pY * ((float) Math.PI / 180F)) * Mth.cos(pX * ((float) Math.PI / 180F));
        float f1 = -Mth.sin((pX + pZ) * ((float) Math.PI / 180F));
        float f2 = Mth.cos(pY * ((float) Math.PI / 180F)) * Mth.cos(pX * ((float) Math.PI / 180F));
        Vec3 delta = new Vec3(f, f1, f2).normalize().multiply(pVelocity, pVelocity, pVelocity).add(pShooter.getDeltaMovement());
        Vec3 pos = pShooter.position().add(delta.scale(0.3).add(0, pShooter.getEyeHeight(), 0));
        this.setDeltaMovement(delta);
        this.setPos(pos);
    }

    public void tick() {
        if (inGroundTime > 1) {
            discard();
        }
        super.tick();
        if (level().isClientSide) {
            spawnParticles();
        }
    }

    @Override
    protected void onHitEntity(@NotNull EntityHitResult pResult) {
        if (level().isClientSide) {
            hitParticles();
        }
        hitEntity(pResult);
    }

    @Override
    protected void onHitBlock(@NotNull BlockHitResult pResult) {
        if (level().isClientSide) {
            hitParticles();
        }
        super.onHitBlock(pResult);
    }

    protected Vec3 particlePos(double dist) {
        return new Vec3(
                level().getRandom().nextIntBetweenInclusive(-100, 100),
                level().getRandom().nextIntBetweenInclusive(-100, 100),
                level().getRandom().nextIntBetweenInclusive(-100, 100)
        ).normalize().scale(dist + ((double) level().getRandom().nextIntBetweenInclusive(0, 100) / 100)).add(position());
    }

    public void spawnParticles() {
    }

    protected void hitEntity(EntityHitResult pResult) {
    }

    protected void hitBlock(BlockHitResult pResult) {
    }

    public void hitParticles() {
    }

    public abstract String name();
}
