package com.jenny.magic.items.wands;

import com.jenny.magic.config.ConfigServer;
import com.jenny.magic.entities.BaseWandProjectile;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.world.level.Level;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

import static com.jenny.magic.Magic.MODID;

public abstract class BaseWand extends Item {
    public BaseWand(Item.Properties properties) {
        super(properties.stacksTo(1).durability(100));
    }

    @Override
    public @NotNull InteractionResultHolder<ItemStack> use(@NotNull Level pLevel, Player pPlayer, @NotNull InteractionHand pUsedHand) {
        ItemStack itemstack = pPlayer.getItemInHand(pUsedHand);
        if (!ConfigServer.C_PREVENT_BREAKING.get() || itemstack.getMaxDamage() - itemstack.getDamageValue() > 1) {
            BaseWandProjectile projectile = newProjectile(pLevel);
            projectile.shootFromRotation(pPlayer, pPlayer.getXRot(), pPlayer.getYRot(), 0.0F, 2.0F);
            pLevel.addFreshEntity(projectile);
            itemstack.hurtAndBreak(1, pPlayer, (player) -> player.broadcastBreakEvent(pPlayer.getUsedItemHand()));
            return InteractionResultHolder.success(itemstack);
        }
        return InteractionResultHolder.pass(itemstack);
    }

    @Override
    public boolean isValidRepairItem(@NotNull ItemStack pStack, @NotNull ItemStack pRepairCandidate) {
        if (pRepairCandidate.getItem().getDescriptionId().equals(Items.DIAMOND.getDescriptionId())) {
            while (pRepairCandidate.getCount() >= 1 && pStack.isDamaged()) {
                pRepairCandidate.shrink(1);
                pStack.setDamageValue(pStack.getDamageValue() - 20);
            }
            return true;
        } else {
            return false;
        }
    }

    abstract BaseWandProjectile newProjectile(Level level);

    @Override
    public void appendHoverText(@NotNull ItemStack pStack, @Nullable Level pLevel, @NotNull List<Component> pTooltipComponents, @NotNull TooltipFlag pIsAdvanced) {
        String key = String.format("tooltip.%s.%s", MODID, this);
        MutableComponent toolTip = Component.translatable(key);
        if (!toolTip.getString().equals(key)) {
            pTooltipComponents.add(toolTip.withStyle(ChatFormatting.DARK_BLUE));
            super.appendHoverText(pStack, pLevel, pTooltipComponents, pIsAdvanced);
        }
    }
}
